package com.persagy.business;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.persagy.entity.HydomcAsset;
import com.persagy.entity.HydomcAssetspec;
import com.persagy.entity.RwdObjectWd;
import com.persagy.enumeration.OperationStatus;
import com.persagy.service.HydomcAssetService;
import com.persagy.service.HydomcAssetspecService;
import com.persagy.utils.CacheUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.ObjectUtils;

import java.util.List;
import java.util.Map;

/**
 * 插入数据处理逻辑
 */
@Slf4j
@Component
public class HandlerData {

    @Autowired
    private CacheUtil cacheUtil;

    @Autowired
    private HydomcAssetService hydomcAssetService;

    @Autowired
    private HydomcAssetspecService hydomcAssetspecService;



    /**
     * @desc 插入数据解析逻辑
     * @param row
     * @param format
     * @return
     */
    public  void  handlerDataToWd(String format,List row ,String dataBaseTableName,String operationStatus,RwdObjectWd rwdObjectWd) throws Exception{
        Map<HydomcAsset,List<HydomcAssetspec>> dataMap =  cacheUtil.handleData(format,row,dataBaseTableName, rwdObjectWd);
        if(ObjectUtils.isEmpty(dataMap)){
            log.info("没有需要插入更新的数据=============");
            return;
        }
        if(OperationStatus.getName(1).equals(operationStatus)){
            //表示插入
            for (Map.Entry<HydomcAsset,List<HydomcAssetspec>> data : dataMap.entrySet()) {
                //保存主表参数表，设备信息主表HYDOMC_ASSET
                HydomcAsset hydomcAsset = data.getKey();
                hydomcAsset.setDatastatus(operationStatus);
                hydomcAssetService.save(hydomcAsset);
                //保存设备参数信息表，设备参数信息HYDOMC_ASSETSPEC
                List<HydomcAssetspec> hydomcAssetspecs =  data.getValue();
                for (HydomcAssetspec hydomcAssetspec:hydomcAssetspecs ) {
                    if(hydomcAssetspec.getAlnvalue()==null && hydomcAssetspec.getNumvalue()==null){
                        //设置信息点为删除状态
                        hydomcAssetspec.setDatastatus("delete");
                    }else {
                        hydomcAssetspec.setDatastatus(operationStatus);
                    }
                }
                hydomcAssetspecService.saveBatch(hydomcAssetspecs);
            }
        }else if(OperationStatus.getName(2).equals(operationStatus)){
            //表示更新，或者删除
            RwdObjectWd rwd = cacheUtil.getEquipId(format,row);
            if(rwd.getValid()==1){
                //表示，有效，只是更新数据
                for (Map.Entry<HydomcAsset,List<HydomcAssetspec>> data : dataMap.entrySet()) {
                    //保存主表参数表，设备信息主表HYDOMC_ASSET
                    HydomcAsset hydomcAsset = data.getKey();
                    hydomcAsset.setDatastatus(operationStatus);
                    UpdateWrapper<HydomcAsset> updateHydomcAssetWrapper = new UpdateWrapper<>();
                    updateHydomcAssetWrapper.eq("SBYBM",hydomcAsset.getSbybm());
                    hydomcAssetService.update(hydomcAsset,updateHydomcAssetWrapper);
                    //保存设备参数信息表，设备参数信息HYDOMC_ASSETSPEC
                    List<HydomcAssetspec> hydomcAssetspecs =  data.getValue();
                    for (HydomcAssetspec hydomcAssetspec:hydomcAssetspecs
                         ) {
                        hydomcAssetspec.setDatastatus(operationStatus);
                        if(hydomcAssetspec.getAlnvalue()==null && hydomcAssetspec.getNumvalue()==null){
                            hydomcAssetspec.setDatastatus("delete");
                        }
                        UpdateWrapper<HydomcAssetspec> updateWrapper = new UpdateWrapper<>();
                        updateWrapper.eq("SITEID",hydomcAssetspec.getSiteid());
                        updateWrapper.eq("SBYBM",hydomcAssetspec.getSbybm());
                        updateWrapper.eq("CLASSSTRUCTUREID",hydomcAssetspec.getClassstructureid());
                        updateWrapper.eq("ASSETATTRID",hydomcAssetspec.getAssetattrid());
                        hydomcAssetspecService.update(hydomcAssetspec,updateWrapper);
                    }
                }

            }else {
                //表示，逻辑删除，数据已经删除，下面的所有信息点的状态都应该是删除
                operationStatus = "delete";
                for (Map.Entry<HydomcAsset,List<HydomcAssetspec>> data : dataMap.entrySet()) {
                    //保存主表参数表，设备信息主表HYDOMC_ASSET
                    HydomcAsset hydomcAsset = data.getKey();
                    hydomcAsset.setDatastatus(operationStatus);
                    UpdateWrapper<HydomcAsset> updateHydomcAssetWrapper = new UpdateWrapper<>();
                    updateHydomcAssetWrapper.eq("SBYBM",hydomcAsset.getSbybm());
                    hydomcAssetService.update(hydomcAsset,updateHydomcAssetWrapper);
                    //保存设备参数信息表，设备参数信息HYDOMC_ASSETSPEC
                    List<HydomcAssetspec> hydomcAssetspecs =  data.getValue();
                    for (HydomcAssetspec hydomcAssetspec:hydomcAssetspecs
                         ) {
                        hydomcAssetspec.setDatastatus(operationStatus);
                        UpdateWrapper<HydomcAssetspec> updateWrapper = new UpdateWrapper<>();
                        updateWrapper.eq("SITEID",hydomcAssetspec.getSiteid());
                        updateWrapper.eq("SBYBM",hydomcAssetspec.getSbybm());
                        updateWrapper.eq("CLASSSTRUCTUREID",hydomcAssetspec.getClassstructureid());
                        updateWrapper.eq("ASSETATTRID",hydomcAssetspec.getAssetattrid());
                        hydomcAssetspecService.update(hydomcAssetspec,updateWrapper);
                    }
                }
            }

        }else {
            //表示删除，但是已经确认中台那边没有物理删除数据的操作，只有逻辑删除的操作，因此这只是留了一个口子，不做任何业务操作
        }
    }



    public static void main(String[] args) {
        String localId = "20.60.60.60.25.0155";
        System.out.println(localId.substring(0,localId.length()-5));
        String tableName = "rwd_object_wd_pj4403070003";
        System.out.println(tableName.substring(tableName.indexOf("pj")));
    }



}
