package com.persagy.ccontinue.impl;

import com.alibaba.fastjson.JSON;
import com.persagy.ccontinue.IPositionHandler;
import com.persagy.ccontinue.config.SyncConfig;
import com.persagy.ccontinue.entity.BinlogPosition;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 默认的端点续传存储器（需要借助分布式存储——此处可以保存到Redis中）
 */
public class DefaultPositionHandler implements IPositionHandler {

    private Map<String, String> cache = new ConcurrentHashMap<>();

    @Override
    public BinlogPosition getPosition(SyncConfig syncConfig) {
        return JSON.parseObject(cache.get(generateKey(syncConfig)), BinlogPosition.class);
    }

    @Override
    public void savePosition(SyncConfig syncConfig, BinlogPosition binlogPosition) {
        cache.put(generateKey(syncConfig), JSON.toJSONString(binlogPosition));
    }


    /**
     * @param syncConfig 参数配置
     * @return 生成的key
     */
    private String generateKey(SyncConfig syncConfig) {
        return syncConfig.getHost() + ":" + syncConfig.getPort();
    }

}