package com.persagy.utils;

import com.alibaba.fastjson.JSONObject;
import com.persagy.configuration.MyBatisPlusConfig;
import com.persagy.service.HydomcAssetService;
import com.persagy.service.HydomcAssetspecService;
import com.persagy.entity.*;
import com.persagy.mapper.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import javax.annotation.PostConstruct;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Component
public class CacheUtil {
    /**
     *  中台与万达的项目关系转换表缓存
     * */
    public static Map<String, VhydomcRelationProjectid> vhydomcRelationProjectidMap = new HashMap<>();

    /**
     *  万达-中台中间关系表缓存，主表驱动关系
     * */
    public static Map<String, VhydomcRwdRelation> vhydomcRwdRelationMap = new HashMap<>();
    /**
     *  万达-中台中间关系表缓存，主表驱动关系
     * */
    public static Map<String,VhydomcRwdRelation> vhydomcRwdRelationLocalIdMap = new HashMap<>();

    /**
     *  万达设备分类V_HYDOMC_CLASSSTRUCTURE缓存
     * */
    public static Map<String, VhydomcClassstructure> vhydomcClassstructureMap = new HashMap<>();

    /**
     *  v_hydomc_asset_relation_rwd map
     * */
    public static Map<String,String> vhydomcAssetRelationRwdMap = new HashMap<>();

    /**
     *  设备参数信息V_HYDOMC_CLASSSPEC map
     * */
    public static Map<String, List<VhydomcClassspec>>  vhydomcClassspecMap = new HashMap<>();

    /**
     * 缓存要同步的classCode，供启动的时第一次全量同步用
     * */
    public  static List<String> classCodeCacheList = new ArrayList<>();

    @Autowired
    private VhydomcRelationProjectidMapper vhydomcRelationProjectidMapper;

    @Autowired
    private VhydomcRwdRelationMapper vhydomcRwdRelationMapper;

    @Autowired
    private VhydomcClassstructureMapper vhydomcClassstructureMapper;

    @Autowired
    private VhydomcAssetRelationRwdMapper vhydomcAssetRelationRwdMapper;

    @Autowired
    private RwdObjectWdMapper rwdObjectWdMapper;

    @Autowired
    private HydomcAssetService hydomcAssetService;

    @Autowired
    private VhydomcClassspecMapper vhydomcClassspecMapper;


    @Autowired
    private HydomcAssetspecService hydomcAssetspecService;


    @PostConstruct
    private void init() {
        log.info("初始化缓存数据开始===========");

        List<VhydomcRelationProjectid>  vhydomcRelationProjectids = vhydomcRelationProjectidMapper.selectList(null);
        for (VhydomcRelationProjectid vhydomcRelationProjectid:vhydomcRelationProjectids
        ) {
            vhydomcRelationProjectidMap.put(vhydomcRelationProjectid.getProjectId().toLowerCase(),vhydomcRelationProjectid);
        }
        log.info("中台与万达的项目id关系缓存===={}",vhydomcRelationProjectidMap);

        List<VhydomcRwdRelation>  vhydomcRwdRelations = vhydomcRwdRelationMapper.selectList(null);
        for (VhydomcRwdRelation vhydomcRwdRelation:vhydomcRwdRelations
        ) {
            if(StringUtils.isEmpty(vhydomcRwdRelation.getLocalId())){
                vhydomcRwdRelationMap.put(vhydomcRwdRelation.getClassCode(),vhydomcRwdRelation);
            }else {
                vhydomcRwdRelationLocalIdMap.put(vhydomcRwdRelation.getLocalId(),vhydomcRwdRelation);
                vhydomcRwdRelationMap.put(vhydomcRwdRelation.getClassCode()+"_"+vhydomcRwdRelation.getLocalId(),vhydomcRwdRelation);
            }
            classCodeCacheList.add(vhydomcRwdRelation.getClassCode());
        }
        log.info("万达-中台中间关系表缓存======={}",vhydomcRwdRelationMap);

        List<VhydomcClassstructure>  vhydomcClassstructures = vhydomcClassstructureMapper.selectList(null);
        for (VhydomcClassstructure vhydomcClassstructure:vhydomcClassstructures
        ) {
            vhydomcClassstructureMap.put(vhydomcClassstructure.getClassstructureid(),vhydomcClassstructure);
        }
        log.info("万达设备分类V_HYDOMC_CLASSSTRUCTURE缓存======={}",vhydomcClassstructureMap);

        List<VhydomcAssetRelationRwd> vhydomcAssetRelationRwds =  vhydomcAssetRelationRwdMapper.selectList(null);
        for (VhydomcAssetRelationRwd vhydomcAssetRelationRwd:vhydomcAssetRelationRwds
        ) {
            vhydomcAssetRelationRwdMap.put(vhydomcAssetRelationRwd.getWdCode(),vhydomcAssetRelationRwd.getRwdCode());
        }
        log.info("万达主表设备参数字段转换配置============{}",vhydomcAssetRelationRwdMap);

        List<VhydomcClassspec> vhydomcClassspecs =  vhydomcClassspecMapper.selectList(null);
        vhydomcClassspecMap = vhydomcClassspecs.stream().collect(Collectors.groupingBy(VhydomcClassspec::getClassstructureid));
        log.info("万达设备参数信息表配置============{}",vhydomcClassspecMap);

        log.info("初始化缓存数据结束===========");
    }

    public Map<HydomcAsset,List<HydomcAssetspec>> handleData(String format,List row ,String dataBaseTableName,RwdObjectWd rwdObjectWdQuery){
        //查询组装数据
        RwdObjectWd rwd ;
        if(row!=null){
             rwd = getEquipId(format,row);
        }else {
            rwd = rwdObjectWdQuery;
        }
        //万达与中台数据种间关系表驱动查询数据判断
        if( StringUtils.isEmpty(rwd.getClassCode()) || !vhydomcRwdRelationMap.containsKey(rwd.getClassCode()) ){
            log.info("监听插入数据classCode={}为空或者万达中台关系表中没有配置该对应的因此该条数据不需要同步",rwd.getClassCode());
            return null;
        }
        //表示需要同步到中台去的数据,先将监听表的infos字段信息查询出来
        String tableName = dataBaseTableName.substring(dataBaseTableName.indexOf(".")+1);
        MyBatisPlusConfig.myTableName.set(tableName);
        RwdObjectWd rwdObjectWd = rwdObjectWdMapper.selectById(rwd.getId());
        //获取万达的广场id
        String wdProjectId = vhydomcRelationProjectidMap.get(tableName.substring(tableName.indexOf("pj"))).getWdProjectId();

        //获取万达中台关系表
        String relationKey = rwdObjectWd.getClassCode();
        if(!StringUtils.isEmpty(rwdObjectWd.getLocalId()) && vhydomcRwdRelationLocalIdMap.containsKey(rwdObjectWd.getLocalId().substring(0,rwdObjectWd.getLocalId().length()-5))){
            relationKey = relationKey+"_"+ rwdObjectWd.getLocalId().substring(0,rwdObjectWd.getLocalId().length()-5);
        }
        VhydomcRwdRelation vhydomcRwdRelation= vhydomcRwdRelationMap.get(relationKey);
        //拿到万达分类id
        String classstructureid = vhydomcRwdRelation.getClassstructureid();
        //信息点&值
        JSONObject infosObject = ObjectUtils.isEmpty(rwdObjectWd.getInfos())?new JSONObject():rwdObjectWd.getInfos();
        HydomcAsset hydomcAsset = assemHydomcAsset(wdProjectId, infosObject);

        //获取万达设备参数配置信息表
        List<VhydomcClassspec> vhydomcClassspecs =  vhydomcClassspecMap.get(classstructureid);
        List<HydomcAssetspec> hydomcAssetspecs = assemHydomcAssetspec(wdProjectId, classstructureid, infosObject, vhydomcClassspecs);
        Map<HydomcAsset,List<HydomcAssetspec>> maps = new HashMap<>();
        maps.put(hydomcAsset,hydomcAssetspecs);
        return maps;
    }

    public  RwdObjectWd getEquipId(String format,List row ){
        JSONObject jsonObject =  JsonUtil.transformationDataToObject(format,row);
        RwdObjectWd rwdObjectWd =  JSONObject.parseObject(jsonObject.toString()).toJavaObject(RwdObjectWd.class);
        return rwdObjectWd;
    }


    /**
     * @desc 组装设备参数信息表，设备参数信息HYDOMC_ASSETSPEC
     * @param wdProjectId
     * @param classstructureid
     * @param infosObject
     * @param vhydomcClassspecs
     * @return
     */
    private List<HydomcAssetspec> assemHydomcAssetspec(String wdProjectId, String classstructureid, JSONObject infosObject, List<VhydomcClassspec> vhydomcClassspecs) {
        List<HydomcAssetspec> hydomcAssetspecLists = new ArrayList<>();
        for (VhydomcClassspec vhydomcClassspec:vhydomcClassspecs
        ) {
            HydomcAssetspec hydomcAssetspec = new HydomcAssetspec();
            //广场id
            hydomcAssetspec.setSiteid(wdProjectId);
            //设备编码
            hydomcAssetspec.setSbybm(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("SBYBM")))?null:(String)infosObject.get(vhydomcAssetRelationRwdMap.get("SBYBM")));
            //设备分类id
            hydomcAssetspec.setClassstructureid(classstructureid);
            //设备分类名称
            hydomcAssetspec.setClassqc(vhydomcClassspec.getClassqc());
            //设备参数id
            hydomcAssetspec.setAssetattrid(vhydomcClassspec.getAssetattrid());
            //设备参数名称
            hydomcAssetspec.setCsdesc(vhydomcClassspec.getCsdesc());
            //设备参数类型
            hydomcAssetspec.setDatatype(vhydomcClassspec.getDatatype());
            //设备参数单位编码
            hydomcAssetspec.setMeasureunitid(vhydomcClassspec.getMeasureunitid());
            //设备参数单位名称
            hydomcAssetspec.setCsjldw(vhydomcClassspec.getCsjldw());
            //设备参数数值
            try {
                hydomcAssetspec.setNumvalue(infosObject.get(vhydomcClassspec.getRwdcode())==null?null:Double.parseDouble(String.valueOf(infosObject.get(vhydomcClassspec.getRwdcode()))));
            } catch (NumberFormatException e) {
                hydomcAssetspec.setAlnvalue(infosObject.get(vhydomcClassspec.getRwdcode())==null?null:String.valueOf(infosObject.get(vhydomcClassspec.getRwdcode())));
                log.error("万达设备参数信息表，设备参数数字值设置失败，因为不是数值型：{}",e);
            }

            //数据状态
//            hydomcAssetspec.setDatastatus("insert");
            //数据更新时间
            hydomcAssetspec.setChangedate(new Date());
            //添加到集合
            hydomcAssetspecLists.add(hydomcAssetspec);
        }
        return hydomcAssetspecLists;
    }

    /**
     * @desc 组装设备信息主表，设备信息主表HYDOMC_ASSET
     * @param wdProjectId
     * @param infosObject
     * @return
     */
    private HydomcAsset assemHydomcAsset(String wdProjectId, JSONObject infosObject) {
        //填充主表数据
        HydomcAsset hydomcAsset = new HydomcAsset();
        //广场id填充
        hydomcAsset.setSiteid(wdProjectId);
        //设备编码
        hydomcAsset.setSbybm(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("SBYBM")))?null:(String)infosObject.get(vhydomcAssetRelationRwdMap.get("SBYBM")));
        //设备名称
        hydomcAsset.setDescription(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("description")))?null:(String)infosObject.get(vhydomcAssetRelationRwdMap.get("description")));
        //设备安装位置
        hydomcAsset.setWzqy(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("wzqy")))?null:(String)infosObject.get(vhydomcAssetRelationRwdMap.get("wzqy")));
        //设备服务区域
        hydomcAsset.setFwqy(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("fwqy")))?null:(String)infosObject.get(vhydomcAssetRelationRwdMap.get("fwqy")));
        //设备型号
        hydomcAsset.setSbxh(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("sbxh")))?null:(String)infosObject.get(vhydomcAssetRelationRwdMap.get("sbxh")));
        //设备分类
        hydomcAsset.setClassstructureid(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("CLASSSTRUCTUREID")))?null:(String)infosObject.get(vhydomcAssetRelationRwdMap.get("CLASSSTRUCTUREID")));
        //设备状态
        hydomcAsset.setStatus(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("STATUS")))?null:(String)infosObject.get(vhydomcAssetRelationRwdMap.get("STATUS")));
        //设备序号
        hydomcAsset.setSbxhh(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("sbxhh")))?null:(String)infosObject.get(vhydomcAssetRelationRwdMap.get("sbxhh")));
        //厂家设备序列号
        hydomcAsset.setCjsbxh(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("CJSBXH")))?null:(String)infosObject.get(vhydomcAssetRelationRwdMap.get("CJSBXH")));
        //生产商
        hydomcAsset.setManufacturer(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("MANUFACTURER")))?null:(String)infosObject.get(vhydomcAssetRelationRwdMap.get("MANUFACTURER")));
        //供应商
        hydomcAsset.setVendor(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("VENDOR")))?null:(String)infosObject.get(vhydomcAssetRelationRwdMap.get("VENDOR")));
        //服务商
        hydomcAsset.setFws(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("FWS")))?null:(String)infosObject.get(vhydomcAssetRelationRwdMap.get("FWS")));
        //采购金额
        hydomcAsset.setPurchaseprice(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("PURCHASEPRICE")))?null:Double.parseDouble(String.valueOf(infosObject.get(vhydomcAssetRelationRwdMap.get("PURCHASEPRICE")))));
        //折旧/使用年限
        hydomcAsset.setZjsynx(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("ZJSYNX")))?null:Double.parseDouble(String.valueOf(infosObject.get(vhydomcAssetRelationRwdMap.get("ZJSYNX")))));
        //生产日期/出厂日期
        hydomcAsset.setScrq(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("SCRQ")))?null:(Date)infosObject.get(vhydomcAssetRelationRwdMap.get("SCRQ")));
        //采购日期
        hydomcAsset.setCgrq(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("CGRQ")))?null:(Date)infosObject.get(vhydomcAssetRelationRwdMap.get("CGRQ")));
        //安装日期
        hydomcAsset.setAzdate(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("AZDATE")))?null:(Date)infosObject.get(vhydomcAssetRelationRwdMap.get("AZDATE")));
        //启用日期
        hydomcAsset.setInstalldate(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("INSTALLDATE")))?null:(Date)infosObject.get(vhydomcAssetRelationRwdMap.get("INSTALLDATE")));
        //质保开始日期
        hydomcAsset.setZbksrq(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("ZBKSRQ")))?null:(Date)infosObject.get(vhydomcAssetRelationRwdMap.get("ZBKSRQ")));
        //质保结束日期
        hydomcAsset.setZbjsrq(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("ZBJSRQ")))?null:(Date)infosObject.get(vhydomcAssetRelationRwdMap.get("ZBJSRQ")));
        //中修周期
        hydomcAsset.setZxzq(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("ZXZQ")))?null:Double.parseDouble(String.valueOf(infosObject.get(vhydomcAssetRelationRwdMap.get("ZXZQ")))));
        //大修周期
        hydomcAsset.setDxzq(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("DXZQ")))?null:Double.parseDouble(String.valueOf(infosObject.get(vhydomcAssetRelationRwdMap.get("DXZQ")))));
        //物业类型
        hydomcAsset.setWylx(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("WYLX")))?null:(String)infosObject.get(vhydomcAssetRelationRwdMap.get("WYLX")));
        //设备管理归属
        hydomcAsset.setSbglgs(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("SBGLGS")))?null:(String)infosObject.get(vhydomcAssetRelationRwdMap.get("SBGLGS")));
        //慧云设备编码
        hydomcAsset.setHysbbm(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("HYSBBM")))?null:(String)infosObject.get(vhydomcAssetRelationRwdMap.get("HYSBBM")));
        //是否慧云使用
        hydomcAsset.setSfhysy(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("SFHYSY")))?null:(String)infosObject.get(vhydomcAssetRelationRwdMap.get("SFHYSY")));
        //是否是主力店
        hydomcAsset.setSfzld(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("SFZLD")))?null:(String)infosObject.get(vhydomcAssetRelationRwdMap.get("SFZLD")));
        //备注
        hydomcAsset.setMark(ObjectUtils.isEmpty(infosObject.get(vhydomcAssetRelationRwdMap.get("MARK")))?null:(String)infosObject.get(vhydomcAssetRelationRwdMap.get("MARK")));
//        //数据状态
//        hydomcAsset.setDatastatus(dataStatus);
        //数据更新时间
        hydomcAsset.setChangedate(new Date());
        return hydomcAsset;
    }
}
